Clean up logic for exiting input loop and scoring.
[open-adventure.git] / misc.c
diff --git a/misc.c b/misc.c
index c4b7e732c6fde159dd598395e5e237b2f063500b..e025c78807f585d76a2db794d2c9cb79a970344e 100644 (file)
--- a/misc.c
+++ b/misc.c
@@ -173,7 +173,7 @@ void fSETPRM(long FIRST, long P1, long P2) {
 #define WORD1X (*wORD1X)
 #define WORD2 (*wORD2)
 #define WORD2X (*wORD2X)
-void fGETIN(FILE *input, long *wORD1, long *wORD1X, long *wORD2, long *wORD2X) {
+bool fGETIN(FILE *input, long *wORD1, long *wORD1X, long *wORD2, long *wORD2X) {
 long JUNK;
 
 /*  Get a command from the adventurer.  Snarf out the first word, pad it with
@@ -185,7 +185,8 @@ long JUNK;
 
 L10:   if(BLKLIN)TYPE0();
        MAPLIN(input);
-       if(input == stdin && feof(stdin)) score(1);
+       if (feof(input))
+           return false;
        WORD1=GETTXT(true,true,true,0);
        if(BLKLIN && WORD1 < 0) goto L10;
        WORD1X=GETTXT(false,true,true,0);
@@ -195,7 +196,7 @@ L12:        JUNK=GETTXT(false,true,true,0);
        WORD2X=GETTXT(false,true,true,0);
 L22:   JUNK=GETTXT(false,true,true,0);
        if(JUNK > 0) goto L22;
-       if(GETTXT(true,true,true,0) <= 0)return;
+       if(GETTXT(true,true,true,0) <= 0)return true;
        RSPEAK(53);
         goto L10;
 }
@@ -723,7 +724,7 @@ L2: ATDWRF=I;
 
 
 
/*  Utility routines (SETBIT, TSTBIT, set_seed_from_time, get_next_lcg_value, randrange, RNDVOC, BUG) */
 
 #undef SETBIT
 long fSETBIT(long BIT) {
@@ -758,13 +759,10 @@ long TSTBIT;
 
 #define TSTBIT(MASK,BIT) fTSTBIT(MASK,BIT)
 
-void set_seed_from_time(void)
+void set_seed(long seedval)
 {
-  /* Use the current system time to get seed the ISO rand() function, from which we get a seed for the LCG. */
-  struct timespec ts;
-  clock_gettime(CLOCK_REALTIME, &ts);
-  srand(ts.tv_nsec);
-  lcgstate.x = (unsigned long) rand() % lcgstate.m;
+       srand(seedval);
+       lcgstate.x = (unsigned long) rand() % lcgstate.m;
 }
 
 unsigned long get_next_lcg_value(void)
@@ -857,8 +855,8 @@ void fBUG(long NUM) {
 #define BUG(NUM) fBUG(NUM)
 #undef MAPLIN
 void fMAPLIN(FILE *OPENED) {
-long I, VAL;
-
+       signed char *cp;
+    
 /*  Read a line of input, from the specified input source,
  *  translate the chars to integers in the range 0-126 and store
  *  them in the common array "INLINE".  Integer values are as follows:
@@ -886,23 +884,24 @@ long I, VAL;
 
        if(MAP2[1] == 0)MPINIT();
 
-       if (!oldstyle && SETUP)
-           fputs("> ", stdout);
-       IGNORE(fgets(INLINE+1,sizeof(INLINE)-1,OPENED));
+       if (!oldstyle && SETUP && OPENED == stdin)
+               fputs("> ", stdout);
+       do {
+               IGNORE(fgets(raw_input,sizeof(INLINE)-1,OPENED));
+       } while
+               /* allow comments in logfiles */
+               (!feof(OPENED) && raw_input[0] == '#');
        if (feof(OPENED)) {
-               if (logfp)
+               if (logfp && OPENED == stdin)
                        fclose(logfp);
        } else {
                if (logfp)
-                       IGNORE(fputs(INLINE+1, logfp));
+                       IGNORE(fputs(raw_input, logfp));
                else if (!isatty(0))
-                       IGNORE(fputs(INLINE+1, stdout));
-               LNLENG=0;
-               for (I=1; I<=sizeof(INLINE) && INLINE[I]!=0; I++) {
-               VAL=INLINE[I]+1;
-               INLINE[I]=MAP1[VAL];
-               if(INLINE[I] != 0)LNLENG=I;
-               } /* end loop */
+                       IGNORE(fputs(raw_input, stdout));
+               for (cp = raw_input; *cp; cp++)
+                       INLINE[cp - raw_input + 1]=MAP1[*cp + 1];
+               LNLENG = (cp - raw_input);
                LNPOSN=1;
        }
 }
@@ -723,7 +724,7 @@ L2: ATDWRF=I;
 
 
-/*  Utility routines (SETBIT, TSTBIT, set_seed_from_time, get_next_lcg_value, randrange, RNDVOC, BUG) */
+/*  Utility routines (SETBIT, TSTBIT, set_seed, get_next_lcg_value, randrange, RNDVOC, BUG) */
 
 #undef SETBIT